<?php
function cnb_get_plan($plans, $name) {
    foreach ($plans as $plan) {
        if ($plan->nickname === $name) {
            return $plan;
        }
    }
    return null;
}

function getProfileEditModal($additional_classes=null, $link_text='Verify your information', $modal_header=null, $data_title='') {
    if (!$modal_header) {$modal_header=$link_text;}
    $url = admin_url('admin.php');
    $full_url =  add_query_arg(
        array(
            'TB_inline' => 'true',
            'inlineId' => 'cnb_admin_page_domain_upgrade_profile',
            'height' => '800'),
        $url );
    printf(
        '<a href="%1$s" title="%2$s" class="thickbox open-profile-details-modal %4$s" id="cnb-button-overview-modal-add-new" onclick="cnb_btn=\'%5$s\'">%3$s</a>',
        $full_url,
        __(esc_html($modal_header)),
        __(esc_html($link_text)),
        esc_attr($additional_classes),
        __(esc_attr($data_title))
    );
}
function cnb_domain_upgrade_overview($domain, $user) {
    // Render upgrade form
    $plans = CnbAppRemotePayment::cnb_remote_get_plans();

    $active_currency = null;
    if ($user && !is_wp_error($user) && isset($user->stripeDetails) && !empty($user->stripeDetails->currency)) {
        $active_currency = $user->stripeDetails->currency;
    }

    $profile_set = false;
    if ($user && !is_wp_error($user) && isset($user->address) && !empty($user->address->country)) {
        $profile_set = true;
    }
    ?>
        <script>
            <?php if (!$profile_set) { ?>
            // Unless a profile hasn't been set yet, in which case, ensure we ask customers for that first
            jQuery(function() {
                jQuery('.button-upgrade').hide();
                jQuery('.open-profile-details-modal').on('click', function() {
                    jQuery('.open-profile-details-modal').hide();
                    jQuery('.button-upgrade').show();
                });
            });
            <?php } else { ?>
            // Hide the "Next" buttons, we already have a profile
            jQuery(function() {
                jQuery('.open-profile-details-modal').hide();
            });
            <?php } ?>

            <?php if ($active_currency) { ?>
            // We already know the currency, so a "select currency" tab menu makes no sense
            jQuery(function() {
                jQuery('.nav-tab-wrapper').hide();
            });
            <?php } ?>
        </script>
    <p>Your domain <strong><?php esc_html_e($domain->name) ?></strong> is currently on the <code><?php esc_html_e($domain->type) ?></code> cloud plan.</p>

    <form id="wp_domain_upgrade" method="post">
        <input type="hidden" name="cnb_domain_id" id="cnb_domain_id" value="<?php esc_attr_e($domain->id) ?>">

        <h2>Select a plan to remove the "Powered by Call Now Button" message from your buttons</h2>

        <h2 class="nav-tab-wrapper">
            <a href="#" data-cnb-currency="eur" class="cnb-currency-select cnb-currency-eur nav-tab<?php if($active_currency !== 'usd') {?> nav-tab-active<?php }?>">Euro (&euro;)</a>
            <a href="#" data-cnb-currency="usd" class="cnb-currency-select cnb-currency-usd nav-tab<?php if($active_currency === 'usd') {?> nav-tab-active<?php }?>">US Dollar ($)</a>
        </h2>
        <div class="cnb-message notice"><p class="cnb-error-message"></p></div>
        <div class="cnb-price-plans">
            <div class="currency-box currency-box-eur cnb-flexbox<?php if($active_currency !== 'usd') {?> currency-box-active<?php }?>">
                <?php $plan = cnb_get_plan($plans, 'powered-by-eur-yearly'); ?>
                <div class="pricebox">
                    <h3 class="yearly">Yearly Plan</h3>
                    <div class="benefit">All branding removed</div>
                    <div class="plan-amount"><span class="currency">€</span><span class="euros">1</span><span class="cents">.67</span><span class="timeframe">/month</span></div>
                    <div class="billingprice">
                        Billed at €19.99 annually
                    </div>
                    <?php getProfileEditModal('button button-primary', 'Next', 'Verify your information', 'powered-by-eur-yearly'); ?>
                    <a class="button button-primary button-upgrade powered-by-eur-yearly" href="#" onclick="cnb_get_checkout('<?php _e($plan->id) ?>')">Upgrade</a>
                </div>

                <?php $plan = cnb_get_plan($plans, 'powered-by-eur-monthly'); ?>
                <div class="pricebox">
                    <h3 class="">Monthly Plan</h3>
                    <div class="benefit">All branding removed</div>
                    <div class="plan-amount"><span class="currency">€</span><span class="euros">4</span><span class="cents">.99</span><span class="timeframe">/month</span></div>
                    <div class="billingprice">
                        Billed at €4.99 monthly
                    </div>
                    <?php getProfileEditModal('button button-secondary', 'Next', 'Verify your information', 'powered-by-eur-monthly'); ?>
                    <a class="button button-secondary button-upgrade powered-by-eur-monthly" href="#" onclick="cnb_get_checkout('<?php _e($plan->id) ?>')">Upgrade</a>
                </div>
            </div>
            <div class="currency-box currency-box-usd cnb-flexbox<?php if($active_currency === 'usd') {?> currency-box-active<?php }?>">
                <?php $plan = cnb_get_plan($plans, 'powered-by-usd-yearly'); ?>
                <div class="pricebox">
                    <h3 class="yearly">Yearly Plan</h3>
                    <div class="benefit">All branding removed</div>
                    <div class="plan-amount"><span class="currency">$</span><span class="euros">1</span><span class="cents">.99</span><span class="timeframe">/month</span></div>
                    <div class="billingprice">
                        Billed at $23.88 annually
                    </div>
                    <?php getProfileEditModal('button button-primary', 'Next', 'Verify your information', 'powered-by-usd-yearly'); ?>
                    <a class="button button-primary button-upgrade powered-by-usd-yearly" href="#" onclick="cnb_get_checkout('<?php _e($plan->id) ?>')">Upgrade</a>
                </div>
                <?php $plan = cnb_get_plan($plans, 'powered-by-usd-monthly'); ?>
                <div class="pricebox">
                    <h3 class="">Monthly Plan</h3>
                    <div class="benefit">All branding removed</div>
                    <div class="plan-amount"><span class="currency">$</span><span class="euros">5</span><span class="cents">.99</span><span class="timeframe">/month</span></div>
                    <div class="billingprice">
                        Billed at $5.99 monthly
                    </div>
                    <?php getProfileEditModal('button button-secondary', 'Next', 'Verify your information', 'powered-by-usd-monthly'); ?>
                    <a class="button button-secondary button-upgrade powered-by-usd-monthly" href="#" onclick="cnb_get_checkout('<?php _e($plan->id) ?>')">Upgrade</a>
                </div>
            </div>
        </div>
    </form>

    <p class="cnb-center">All <u>cloud</u> plans (free and paid) contain the following features:</p>
    <div  class="cnb-center" style="margin-bottom:50px;">
        <div><b>&check;</b> Phone, Email, Location, WhatsApp, Links</div>
        <div><b>&check;</b> Unlimited buttons</div>
        <div><b>&check;</b> Multibutton</div>
        <div><b>&check;</b> Buttonbar (full width with multiple actions)</div>
        <div><b>&check;</b> Advanced page targeting options</div>
        <div><b>&check;</b> Scheduling</div>
    </div>
<?php }
