<?php

require_once dirname( __FILE__ ) . '/CnbAdminNotices.class.php';
require_once dirname( __FILE__ ) . '/../admin/api/CnbAdminCloud.php';

function cnb_upgrade_notice($cnb_old_version, $cnb_changelog) {

    $message =  '<h3>' . CNB_NAME . ' has been updated!</h3><h4>What\'s new?</h4>';
    // Only on first run after update show list of changes since last update
    foreach ( $cnb_changelog as $key => $value ) {
        if ( $key > $cnb_old_version ) {
            $message .= '<h3>' . esc_html($key) . '</h3>';
            if ( is_array( $value ) ) {
                foreach ( $value as $item ) {
                    $message .= '<p><span class="dashicons dashicons-yes"></span> ' . esc_html($item) . '</p>';
                }
            } else {
                $message .= '<p><span class="dashicons dashicons-yes"></span> ' . esc_html($value) . '</p>';
            }
        }
    }

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message, 'cnb_update_'.$cnb_old_version);

}

function cnb_settings_get_account_missing_notice() {
    $message = '<h3 class="title">Activating Premium</h3>
            <p>To activate Call Now Button Premium, you\'ll need an account at
                <a href="https://app.callnowbutton.com?utm_source=wp-plugin&utm_medium=referral&utm_campaign=beta_tester&utm_term=sign-up-for-api">https://app.callnowbutton.com</a>, generate a unique key and paste that into the field below. There\'s 2 ways to do this:</p>
            <h4>Option 1: The fast track</h4>
            <a class="button button-primary" id="cnb-go-create-account" href="">Take the fast track</a>
            <p>In the fast track we\'ve tried to automate as much as possible of the process and make this as easy as possible.</p>
            <h4>Option 2: The manual process</h4>
            <ol>
                <li>Create your account at <a href="https://app.callnowbutton.com?utm_source=wp-plugin&utm_medium=referral&utm_campaign=beta_tester&utm_term=sign-up-for-api">https://app.callnowbutton.com</a></li>
                <li>Go to your profile info by clicking on the user icon in the top right corner and then click <strong>Create new API key</strong>.</li>
                <li>Copy the API key that appears, paste it into the field below and click <strong>Save API key</strong>.</li>
            </ol>';
    $message .= cnb_settings_api_key_input();

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message);
}

function cnb_settings_api_key_invalid_notice() {
    $message = '<h3 class="title">Ooops, that API key doesn\'t seem right</h3>
            <p>The saved API key is invalid. Let\'s give it another try:</p>
            <ol>
                <li>Head over to the profile panel on <a href="https://app.callnowbutton.com/app/profile" target="_blank">app.callnowbutton.com</a>.</li>
                <li>Click <strong>Create new API key</strong>, enter a unique name (can be anything. I always enter my website domain here) and click <strong>Generate new API key</strong>.</li>
                <li>Copy the API key, paste it into the field below and click <strong>Save key</strong>.</li>
            </ol>
            <p>If it\'s still not working, we might be experiencing server issues. Please wait a few minutes and try again. You can check our <a target="_blank" href="https://status.callnowbutton.com">status page</a> to be sure.</p>';
    $message .= cnb_settings_api_key_input();

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message);
}

function cnb_generic_error_notice($user) {
    $message = '<h3 class="title">Something went wrong!</h3>
            <p>Something has gone wrong and we do not know why...</p>
            <p>As unlikely as it is, our service might be experiencing issues (check <a href="https://status.callnowbutton.com">our status page</a>).</p>
            <p>If you think you\'ve found a bug, please report it at our <a href="https://callnowbutton.com/support/" target="_blank">Help Center</a>.';
    $message .= CnbAdminCloud::cnb_admin_get_error_message_details($user);

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message);
}

function cnb_settings_api_key_input() {
    // Create URL
    $url = admin_url('admin.php');
    $settings_url = esc_url( $url );

    $message = '<form method="post" action="' . esc_url( admin_url('options.php') ) . '" class="cnb-container">';
    ob_start();
    settings_fields('cnb_options');
    $message .= ob_get_clean();
    $message .= '<input type="hidden" name="page" value="call-now-button-settings" />
            <table class="form-table">
                <tr class="when-cloud-enabled">
                <th scope="row">API key</th>
                <td>
                    <input type="hidden" id="admin_email" value="'. esc_attr(get_bloginfo('admin_email')) .'">
                    <input type="hidden" id="admin_url" value="'. esc_attr($settings_url) .'">
                    <input type="text" class="regular-text" name="cnb[api_key]"
                           placeholder="e.g. b52c3f83-38dc-4493-bc90-642da5be7e39"/>
                    '. get_submit_button(__('Save API key'), 'primary', 'submit', false).'
                </td>
                </table>
            </form>';
    return $message;
}

function cnb_settings_get_domain_missing_notice($domain) {
    $message = '<h3 class="title">Domain not found yet</h3>
                <p>You have enabled Call Now Button Premium and you are logged in. Now we need to create this domain remotely.</p>
                <p>
                <form action="' . esc_url( admin_url('admin-post.php') ) . '" method="post">
                    <input type="hidden" name="page" value="call-now-button-settings" />
                    <input type="hidden" name="action" value="cnb_create_cloud_domain" />
                    <input type="hidden" name="_wpnonce" value="' . wp_create_nonce('cnb_create_cloud_domain') .'" />
                    '. get_submit_button(__('Create domain'), 'secondary', 'submit', false).'
                </form>
                </p>';
    $message .= CnbAdminCloud::cnb_admin_get_error_message_details( $domain );

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message);
}

function cnb_settings_get_button_missing_notice() {
    $message = '<h3 class="title">Creating your first button</h3>
            <p>You have enabled Call Now Button Premium and your domain have been set up.
            Now it\'s time to create your first button.</p>
            <p>To make it easy, we can migrate your existing button to your account.</p>
            <p><form action="'. esc_url( admin_url('admin-post.php') ) .'" method="post">
                <input type="hidden" name="page" value="call-now-button-settings" />
                <input type="hidden" name="action" value="cnb_migrate_legacy_button" />
                <input type="hidden" name="_wpnonce" value="'. wp_create_nonce('cnb_migrate_legacy_button') .'" />
                '. get_submit_button(__('Migrate button'), 'secondary', 'submit', false).'
            </form></p>';

    $notice = new CnbNotice('warning', $message);
    $notice->dismiss_option = 'cnb_settings_get_button_missing_notice';
    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->notice($notice);
}

function cnb_settings_get_buttons_missing_notice($error) {
    $message = '<h3 class="title">Could not retrieve Buttons</h3>
            <p>Something unexpected went wrong retrieving the Buttons for this API key</p>';
    $message .= CnbAdminCloud::cnb_admin_get_error_message_details( $error );

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message);
}

function cnb_api_key_invalid_notice($error) {
    $url = admin_url('admin.php');
    $redirect_link =
        add_query_arg(
            array(
                'page' => 'call-now-button-settings',
            ),
            $url );
    $redirect_url = esc_url( $redirect_link );

    $message = '<h3 class="title">API Key invalid</h3>
            <p>You have enabled Call Now Button Premium, but you still need a valid API key.</p>
            <p>Go to <a href="'.$redirect_url.'">Settings</a> for instructions.</p>';
    $message .= CnbAdminCloud::cnb_admin_get_error_message_details( $error );

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->renderError($message);

}

function cnb_button_disabled_notice() {
    $url = admin_url('admin.php');
    $redirect_link =
        add_query_arg(
            array(
                'page' => 'call-now-button',
            ),
            $url );
    $redirect_url = esc_url( $redirect_link );

    $message = '<p>Your button is currently <strong>not enabled</strong>. Click <a href="'.$redirect_url.'">here</a> to enable your button.</p>';

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message);
}

function cnb_button_classic_enabled_but_no_number_notice() {
    $url = admin_url('admin.php');
    $redirect_link =
        add_query_arg(
            array(
                'page' => 'call-now-button',
            ),
            $url );
    $redirect_url = esc_url( $redirect_link );

    $message = '<p>The Call Now Button is currently <strong>active without a phone number</strong>.
        Change the <i>Button status</i> under <a href="'.$redirect_url.'">My button</a> to disable or enter a phone number.</p>';

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->warning($message);
}

function cnb_caching_plugin_warning_notice($caching_plugin_name) {
    $message = '<p><span class="dashicons dashicons-warning"></span>
        Your website is using a <strong><i>Caching Plugin</i></strong> (' . $caching_plugin_name . ').
        If you\'re not seeing your button or your changes, make sure you empty your cache first.</p>';

    $adminNotices = CnbAdminNotices::get_instance();
    $adminNotices->error($message);
}
